/*
  Copyright 2020 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
package com.kakao.sdk.partner.sample

import android.os.Bundle
import android.widget.Toast
import androidx.appcompat.app.AppCompatActivity
import com.kakao.sdk.common.util.SdkLog
import com.kakao.sdk.partner.friend.client.PickerClient
import com.kakao.sdk.partner.friend.model.*
import com.kakao.sdk.partner.sample.databinding.ActivityChatPickerSettingBinding

class ChatPickerSettingActivity : AppCompatActivity() {
    private lateinit var binding: ActivityChatPickerSettingBinding
    private var title: String? = null
    private var chatFilter: List<PickerChatFilter>? = null
    private var viewAppearance: ViewAppearance? = null
    private var orientation: PickerOrientation? = null
    private var disableSelectOptions: List<DisableSelectOption>? = null
    private var maxPickableCount: Int = 30
    private var minPickableCount: Int = 1

    private val disableCustoms = mutableListOf<DisableSelectOption>()

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        binding = ActivityChatPickerSettingBinding.inflate(layoutInflater)
        setContentView(binding.root)

        val orientationArray = resources.getStringArray(R.array.picker_orientation)
        val viewAppearanceArray = resources.getStringArray(R.array.view_appearance)

        binding.apply {
            disableSelectClearBtn.setOnClickListener {
                disableCustoms.clear()
                disableCustomCount.text = "${disableCustoms.size}"
            }

            disableSelectAddBtn.setOnClickListener {
                val message = disableMessageEt.text.toString()
                val uuidText = uuidEt.text.toString()
                val uuids = if (uuidText.isEmpty()) {
                    null
                } else {
                    uuidText.split(",").map { it.trim() }.toList()
                }
                disableCustoms.add(DisableSelectOption(DisableSelectReason.CUSTOM, message, uuids))

                disableMessageEt.text.clear()
                uuidEt.text.clear()
                disableCustomCount.text = "${disableCustoms.size}"
            }

            cancelBtn.setOnClickListener {
                finish()
            }

            createBtn.setOnClickListener {
                title = getPickerTitle()
                maxPickableCount = getMaxPickableCount()
                minPickableCount = getMinPickableCount()
                chatFilter = getPickerChatFilter()
                viewAppearance = getViewAppearance(viewAppearanceArray)
                orientation = getPickerOrientation(orientationArray)
                disableSelectOptions = listOf()
                disableSelectOptions = getDisableSelect()

                val callback =
                    { selectedUsers: SelectedUsers?, selectedChat: SelectedChat?, error: Throwable? ->
                        if (error != null) {
                            SdkLog.e("채팅방 정보 불러오기 실패 $error")
                            Toast.makeText(
                                this@ChatPickerSettingActivity,
                                "채팅방 정보 불러오기 실패 $error",
                                Toast.LENGTH_LONG
                            ).show()
                        } else {
                            val message: String = if (chatMemberCb.isChecked) {
                                "채팅방 멤버 정보 불러오기 성공 $selectedUsers"
                            } else {
                                "채팅방 정보 불러오기 성공 $selectedChat"
                            }
                            SdkLog.i(message)
                            Toast.makeText(
                                this@ChatPickerSettingActivity,
                                message,
                                Toast.LENGTH_LONG
                            ).show()
                        }
                    }

                val param = getPickerRequestParams()
                if (popupCb.isChecked) {
                    PickerClient.instance.selectChatPopup(
                        this@ChatPickerSettingActivity,
                        param,
                        callback
                    )
                } else {
                    PickerClient.instance.selectChat(
                        this@ChatPickerSettingActivity,
                        param,
                        callback
                    )
                }
            }
        }
    }

    private fun ActivityChatPickerSettingBinding.getDisableSelect(): List<DisableSelectOption>? {
        val filter = mutableListOf<DisableSelectOption>()
        if (binding.msgBlockedCb.isChecked) filter.add(DisableSelectOption(DisableSelectReason.MSG_BLOCKED))
        if (binding.registeredCb.isChecked) filter.add(DisableSelectOption(DisableSelectReason.REGISTERED))
        if (binding.unregisteredCb.isChecked) filter.add(DisableSelectOption(DisableSelectReason.UNREGISTERED))
        if (binding.notFriendCb.isChecked) filter.add(DisableSelectOption(DisableSelectReason.NOT_FRIEND))
        filter.addAll(disableCustoms)
        return if (filter.isEmpty()) null else filter
    }

    private fun ActivityChatPickerSettingBinding.getPickerOrientation(
        orientationArray: Array<String>
    ) = when (orientationSpinner.selectedItem) {
        orientationArray[1] -> PickerOrientation.AUTO
        orientationArray[2] -> PickerOrientation.PORTRAIT
        orientationArray[3] -> PickerOrientation.LANDSCAPE
        else -> null
    }

    private fun ActivityChatPickerSettingBinding.getViewAppearance(
        viewAppearanceArray: Array<String>
    ) = when (themeSpinner.selectedItem) {
        viewAppearanceArray[1] -> ViewAppearance.AUTO
        viewAppearanceArray[2] -> ViewAppearance.LIGHT
        viewAppearanceArray[3] -> ViewAppearance.DARK
        else -> null
    }

    private fun ActivityChatPickerSettingBinding.getPickerChatFilter(): List<PickerChatFilter>? {
        val filter = mutableListOf<PickerChatFilter>()
        if (binding.directCb.isChecked) filter.add(PickerChatFilter.DIRECT)
        if (binding.multiCb.isChecked) filter.add(PickerChatFilter.MULTI)
        if (binding.regularCb.isChecked) filter.add(PickerChatFilter.REGULAR)
        if (binding.openCb.isChecked) filter.add(PickerChatFilter.OPEN)
        return if (filter.isEmpty()) null else filter
    }

    private fun ActivityChatPickerSettingBinding.getMinPickableCount() =
        if (minCountEt.text.toString().isBlank()) {
            minCountEt.hint.toString().toInt()
        } else {
            minCountEt.text.toString().toInt()
        }

    private fun ActivityChatPickerSettingBinding.getMaxPickableCount() =
        if (maxCountEt.text.toString().isBlank()) {
            maxCountEt.hint.toString().toInt()
        } else {
            maxCountEt.text.toString().toInt()
        }

    private fun ActivityChatPickerSettingBinding.getPickerTitle() =
        if (titleEt.text.toString().isEmpty()) {
            null
        } else {
            titleEt.text.toString()
        }

    private fun getPickerRequestParams(): PickerChatRequestParams {
        return binding.run {
            val selectionType =
                if (chatMemberCb.isChecked) PickerChatSelectionType.CHAT_MEMBER else PickerChatSelectionType.CHAT

            PickerChatRequestParams(
                selectionType = selectionType,
                title = title,
                chatFilters = chatFilter,
                viewAppearance = viewAppearance,
                orientation = orientation,
                enableSearch = searchbarCb.isChecked,
                displayAllProfile = allProfileCb.isChecked,
                disableSelectOptions = disableSelectOptions,
                maxPickableCount = maxPickableCount,
                minPickableCount = minPickableCount
            )
        }
    }

    companion object {
        const val TAG = "KakaoSDKSample"
    }
}